/**********************************************************************************************
    Program: BURST.C   
	Purpose: Photon burst analysis/FCS/PCH
	Author:  W.R. Zipfel
	Date:    11/26/2000 - 12/2004 ver 1.0- 1.9
	revised  1/11/2008 ver 2.0 - added Correlator.com file support
	revised  7/01/08 - 9/1/08  ver 3.0 - complete rewrite   
 ***********************************************************************************************/
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>    
#include "Burst.h"
#include "resource.h"

char szAppName[] = "Burst";     // Window class Name

/**********************************************************************************************
  WinMain
 ********************************************************************************************/
int PASCAL WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,LPSTR lpCmdLine, int nCmdShow)
{
  MSG       msg;
  WNDCLASS  wc;
  HWND      hwnd;
  char szWindowTitle[] = "Burst.exe - Photon Stream Analysis";  // Base string for main window title
  
  wc.style = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc = WndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInstance;
  wc.hIcon = LoadIcon(hInstance, "BURST_ICON");
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);
  wc.lpszMenuName = "MAINMENU";		
  wc.lpszClassName = szAppName;

  hInst = hInstance;                            // save a global copy of the instance handle
  if(!CheckScreenSize()) return 0;              // exits if display is < 1024 x 768, otherwise returns main window coordinates
  if(!RegisterClass(&wc)) return 0;             // bail out, but this never happens anymore
  if(!LoadInitFile()) CreateIniFile();          // if burst.ini is not found create a new one with default values
  InitGlobals();                                // set values for various program global variables
  InitCommonControls();                         // load the commmon control lib - comctl32.lib must be in project

  hwnd = CreateWindow(szAppName, szWindowTitle, WS_OVERLAPPEDWINDOW, mw.left, mw.top, 
	                  mw.right, mw.bottom, NULL, NULL, hInstance, NULL);
  SetupMainWindow(hwnd);
  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  while (GetMessage (&msg, NULL, 0, 0))    {
        if (!TranslateAccelerator (hwnd, hAccel, &msg))  {
               TranslateMessage (&msg) ;
               DispatchMessage (&msg) ;
        }
     }
  return msg.wParam ;
}


/**************************************************************************************************
 Main window procedure
 *************************************************************************************************/
LRESULT CALLBACK WndProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{

  switch (message) {
      case WM_CREATE:
			hAccel =  LoadAccelerators(hInst, "KEY_ACCEL");
		    return 0;
      case WM_HSCROLL :   // position scroll
		    if(lParam == (LPARAM) GetDlgItem(hPSControls, IDC_POSITION_TB)) CountsPositionTrackBar();
		    if(lParam == (LPARAM) GetDlgItem(hPSControls, IDC_SEGTIME_SLIDER)) SegmentTimeTrackBar();
   		    return 0; 

      case WM_COMMAND: 
			switch (LOWORD (wParam)) {
           	  case IDM_OPENDATASET:  LoadDataFile(hwnd, active_ps); break;
              case IDM_AUTOCORR:     AutoCorrelate(hwnd, active_ps); break;
			  case IDM_CROSSCORR:    CrossCorrelate(hwnd, active_ps); break;  // active set = A as in AxB
			  case IDM_SEG_AUTOCORR: SegmentedAutocorrelation(hwnd, active_ps); break;
			  case IDM_ANALYZE:      BurstAnalysis(hwnd, active_ps); break;
	          case IDM_CLOSEDATASET: CloseDataSet(active_ps); break;
			  case IDM_SHOWDATA:     ShowDataTable(active_ps); break;
			  case IDM_SAVESEGMENT:  SaveSegment(hwnd, active_ps);  break;
    	      case IDM_ABOUT:        DialogBox(hInst, "AboutBox", hwnd, (DLGPROC)AboutProc); break;
        	  case IDM_SAVEDATASET:  SaveDataSetAsPHCFile(hwnd, active_ps); break;
			  case IDM_SAVEBURSTRESULTS: SaveHistResults(hwnd, active_ps);      break;
			  case IDM_SAVEBURSTDATA: SaveBurstResults(hwnd, active_ps);     break;
			  case IDM_SAVE_FCSCURVE: SaveFCSCurve(hwnd, active_ps); break;
              case IDM_HELPINDEX:    HelpSystemDialog(hwnd); break; 
			  case IDM_COUNTSSETUP:  DialogBox(hInst,"COUNTSSETUP", hwnd, (DLGPROC)CountsSetupProc); break;
  			  case IDM_FCSSETUP:     DialogBox(hInst,"FCSSETUP", hwnd, (DLGPROC)FCSSetupProc); break;
			  case IDM_HISTSETUP:    DialogBox(hInst,"HISTSETUP", hwnd, (DLGPROC)HistSetupProc); break;
			  case IDM_PRINT_SETUP:  WGPrinterSetup (NULL);   break;
			  case IDM_PRINT_PAGE :  WGPrintPageEx(pResultsPage, 0.05, 0.15, 0.95, 0.95); break;
			  case IDM_PRINT_PARAM : WGPrintOptionsDlg ();  break;
			  case IDM_EXIT:         SendMessage(hwnd, WM_CLOSE, 0,0); break;
			  case IDC_PCH:          DisplayPCHPlot(hwnd, active_ps); break;
			  case IDM_STEPBACKWARD: StepPositionTB(IDM_STEPBACKWARD); break;
			  case IDM_STEPFORWARD:  StepPositionTB(IDM_STEPFORWARD); break;
			  case IDC_BURST_NEXT:   MoveBurstMarker(IDC_BURST_NEXT, active_ps); break;
		 	  case IDC_BURST_BACK:   MoveBurstMarker(IDC_BURST_BACK, active_ps); break;
			  case IDC_SHOWBURSTMARKERS: ToggleBurstMarkerFlag(); break; 
			  case IDC_PS1:          SetActivePS(0);	break;
			  case IDC_PS2:          SetActivePS(1); break;
			  case IDM_BATCHPROCESS: DialogBox(hInst,"BATCHDIALOG", hwnd, (DLGPROC)BurstBatchProc); break;
			  case IDM_SHOWRESULTSWINDOW: ToggleResultsWindow(hwnd); break;
			  case IDC_ESTIMATEBACKGROUND: EstimateBackground(GETPS(active_ps)) ; break; 
			  case IDC_SMOOTHDATA:    SGSmoothData(active_ps); break;
              case IDC_AUTOSCALEY:    ToggleCountsAutoScale(); break;
              case IDC_HISTAUTOSCALE: ToggleHistAutoScale(); break; 
			  case IDM_PS1_MINUS_C1:  DoPSMath(IDM_PS1_MINUS_C1); break;    // subtract C1 from PS1
			  case IDM_PS1_PLUS_C1:   DoPSMath(IDM_PS1_PLUS_C1); break;     // add C1 to PS1
			  case IDM_PS2_MINUS_C2:  DoPSMath(IDM_PS2_MINUS_C2); break;    // subtract C2 from PS2
			  case IDM_PS2_PLUS_C2:   DoPSMath(IDM_PS2_PLUS_C2); break;     // add C2 to PS2
			  case IDM_PS1_A1_X_PS1:  DoPSMath(IDM_PS1_A1_X_PS1); break;    // mult PS1 by a1
			  case IDM_PS2_A2_X_PS2:  DoPSMath(IDM_PS2_A2_X_PS2); break;    // mult PS2 by a2
			  case IDM_PS1_PLUS_PS2:  DoPSMath(IDM_PS1_PLUS_PS2); break;    // PS1 = PS1 + PS2
  			  case IDM_PS1_MINUS_PS2: DoPSMath(IDM_PS1_MINUS_PS2); break;   // PS1 = PS1 - PS2
			  case IDM_PS1_ADD_PNOISE: DoPSMath(IDM_PS1_ADD_PNOISE); break;  // PS1 = PS1 + Pois noise
  			  case IDM_PS2_ADD_PNOISE: DoPSMath(IDM_PS2_ADD_PNOISE); break;  // PS2 = PS2 + Pois noise
				  
			  case ID_MATH_BLEEDTHROUGHCORRECTION:
				   if(flags.bSet[0] && flags.bSet[1]) {
					  BleedThruCorrectDataSets(&ps1, &ps2, init.f1, init.f2);
				      DisplayData();
				   }
		  		   break;

		      case IDM_SELECTBACKGROUND:
				   if(flags.bSet[0] || flags.bSet[1]) {
					  if(fCountsMouse == FIND_BACKGROUND) {
						 TurnOffBackGroundSelect(hwnd);
					  }
					  else {    
					     CheckMenuItem(GetMenu(hwnd), IDM_SELECTBACKGROUND, MF_CHECKED);
						 fCountsMouse = FIND_BACKGROUND;
					  }
				   }
				   break;
	   
			  case IDC_SELECT_AC_REGION:
				   if(flags.bSet[0] || flags.bSet[1]) {
					  if(fCountsMouse == SET_AC_LIMIT) {
					     TurnOffFCSSegmentSelect(hwnd);
					  }
					  else {    
					     CheckMenuItem(GetMenu(hwnd), IDC_SELECT_AC_REGION, MF_CHECKED);
						 fCountsMouse = SET_AC_LIMIT;
					  }
				   }
				   break;
			}
            return 0;

       case WM_DESTROY:
		    GetUserSetParameters(); 
		    SaveInitFile();
		    FreePhotonStreamMem(&ps1);
		    FreePhotonStreamMem(&ps2);
	        FreeBurstMem(&burst1);
		    FreeBurstMem(&burst2);
     	    WGCleanup(FALSE); 		
			PostQuitMessage(0);
	        return 0;
    }
    return (DefWindowProc(hwnd, message, wParam, lParam));
}


/*****************************************************************************************
  Initializes global variables 
 ***************************************************************************************/
void InitGlobals(void)
{
  int i;

  flags.bSet[0] = 0;
  flags.bSet[1] = 0;
  active_ps = 0;                        
  flags.bResultsReady[0] = FALSE;
  flags.bResultsReady[1] = FALSE;
  flags.bResultsSaved[0] = FALSE;
  flags.bResultsSaved[1] = FALSE;
  bShowResultsWin = FALSE; 
  init.f1 = init.rab / (init.rab + 1.0); 
  init.f2 = init.rba / (init.rba + 1.0); 
  cd.Displaymode = init.Displaymode; 
  cd.binwidth = init.BinWidth; 
  cd.bShowBurstMarkers = TRUE; 

  // reserve memory for QC plotting arrays
  cd.n = QC_SIZE;   // QC_SIZE is defined in burst.h as 1000
  cd.x = (double *) calloc(cd.n, sizeof(double));
  cd.y1 = (double *) calloc(cd.n, sizeof(double));
  cd.y2 = (double *) calloc(cd.n, sizeof(double));
  for(i=0;i<3;i++) {
      pd[i].x = (double *) calloc(cd.n, sizeof(double));
      pd[i].y = (double *) calloc(cd.n, sizeof(double));
  }
  pdFCS.x = (double *) calloc(cd.n, sizeof(double));
  pdFCS.y = (double *) calloc(cd.n, sizeof(double));
}

/*****************************************************************************************
  Sets initial window sizes and positions
 ***************************************************************************************/
void SetupMainWindow(HWND hwnd)
{
  RECT r, cr;
  int cyHeight;
  char szBuf[256];

  GetClientRect(hwnd, &cr);
  
  hPSControls = CreateDialog(hInst, "PHOTONVIEWER", hwnd,  WndProc);  // modeless dialogbox for a control panel
  GreyOutBurstButtons(TRUE);
  GreyOutControls(TRUE);

  SendDlgItemMessage(hPSControls, IDC_POSITION_TB, TBM_SETRANGE, (WPARAM) TRUE, (LPARAM) MAKELONG(0, 100)); 
  SendDlgItemMessage(hPSControls, IDC_POSITION_TB, TBM_SETPOS,   (WPARAM) TRUE, (LPARAM) 0); 
  SendDlgItemMessage(hPSControls, IDC_POSITION_TB, TBM_SETTICFREQ, (WPARAM) 10, (LPARAM) 0); 
  
  SetUpBinWidths(hPSControls, init.BinWidth);

  SendDlgItemMessage(hPSControls, IDC_SEGTIME_SLIDER, TBM_SETRANGE, (WPARAM) TRUE, (LPARAM) MAKELONG(0, 100)); 
  SendDlgItemMessage(hPSControls, IDC_SEGTIME_SLIDER, TBM_SETPOS,   (WPARAM) TRUE, (LPARAM) 0); 
  SendDlgItemMessage(hPSControls, IDC_SEGTIME_SLIDER, TBM_SETTICFREQ, (WPARAM) 10, (LPARAM) 0); 

  UpdateCPSEditbox();
  sprintf(szBuf,"%s",(int) "- - - ");
  SendDlgItemMessage(hPSControls, IDC_POSITION, WM_SETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));
  SendDlgItemMessage(hPSControls, IDC_BURSTNUM, WM_SETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));
  SendDlgItemMessage(hPSControls, IDC_BURSTINFO, WM_SETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));
   
  if(init.bAutoScale) CheckDlgButton(hPSControls, IDC_AUTOSCALEY, BST_CHECKED);
  
  if(init.MinCountsPerBurst < init.TriggerCounts) init.MinCountsPerBurst = init.TriggerCounts;
  sprintf(szBuf,"%d",(int) init.MinCountsPerBurst);
  SendDlgItemMessage(hPSControls,IDC_MINCOUNTS, WM_SETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));
  sprintf(szBuf,"%u", init.BackGrdCounts);
  SendDlgItemMessage(hPSControls,IDC_BACKGRDCOUNTS, WM_SETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));
  sprintf(szBuf,"%d",(int) init.SubMinBins);
  SendDlgItemMessage(hPSControls,IDC_SUBMINBINS, WM_SETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));
  sprintf(szBuf,"%d",init.TriggerCounts);
  SendDlgItemMessage(hPSControls,IDC_TRIGGER, WM_SETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));

  SetSGFilterWidths(hPSControls, init.SGFilterWidth);
  
  if(init.bBackgroundSubtract) CheckDlgButton(hPSControls, IDC_AUTOSUBTRACT, BST_CHECKED);
  if(init.bHistAutoScale) CheckDlgButton(hPSControls, IDC_HISTAUTOSCALE, BST_CHECKED);
  if(init.bShowBurstMarkers) CheckDlgButton(hPSControls, IDC_SHOWBURSTMARKERS, BST_CHECKED); 

  GetWindowRect(hPSControls, &r);
  cyHeight = r.bottom - r.top;

  MoveWindow(hPSControls, 0, cr.bottom - cyHeight, cr.right - cr.left, cyHeight,  TRUE);  
  ShowWindow(hPSControls, SW_SHOW);

  WGCreatePage("page1", hwnd,  hInst, "", InitCountsPage,  NULL, C_WHITE, MM_PROPORT, WS_CHILD, 
	            PAGE_EXACT, cr.left, cr.top, cr.right - cr.left, cr.bottom - cyHeight); 

  // create the results display window
  pResultsPage = WGCreatePage("page2", hwnd,  hInst, "PS Analysis Results", (void *) InitResultsPage,  "RESULTSPAGEMENU", C_WHITE, 
	             MM_PROPORT, WS_POPUP|WS_CAPTION|WS_THICKFRAME|WS_CLIPCHILDREN,
                 PAGE_EXACT, rw.left, rw.top, rw.right, rw.bottom); 
  
  ShowWindow(WGGetPageWindow(pResultsPage), SW_HIDE);    // hide window
}

/*****************************************************************************************
  Check screen resolution and if acceptable fill window RECT structures with optimal coordinates
 ***************************************************************************************/
BOOL CheckScreenSize(void)
{
  int cxWidth, cyHeight;
  double aspect_ratio, scale_factor; 
  RECT r;

  cxWidth = GetSystemMetrics(SM_CXSCREEN);
  cyHeight = GetSystemMetrics(SM_CYSCREEN);

  if(cxWidth < 1024 || cyHeight < 768) {
	  MessageBox(NULL, "Program requires 1024 x 768 of larger screen resolution.", "Display error", MB_OK); 
	  return(FALSE);
  }

  aspect_ratio = (double) cxWidth / (double) cyHeight; 
  scale_factor = (aspect_ratio < 1.59)? 0.50 : 0.40; 

  SystemParametersInfo(SPI_GETWORKAREA, 0, (PVOID) &r, 0);    
  cyHeight = r.bottom - r.top;   // size of display minus the start bar

  mw.left = 0;
  mw.right = cxWidth;
  mw.top = 0;
  mw.bottom = (int) (scale_factor * (double) cyHeight);

  rw.left = 0;
  rw.right = cxWidth;
  rw.top = (int) (scale_factor * (double) cyHeight);
  rw.bottom = (int) ((1.0 - scale_factor) * (double) cyHeight);

  return(TRUE);
}
  

 /*******************************************************************************************************************
  Greys and inactivates burst related controls  - used if burst are not analysed yet.
 ******************************************************************************************************************/
void GreyOutBurstButtons(BOOL bState)
{
	if(bState) {
      EnableWindow(GetDlgItem(hPSControls, IDC_BURST_BACK), FALSE);
      EnableWindow(GetDlgItem(hPSControls, IDC_BURST_NEXT), FALSE);
      EnableWindow(GetDlgItem(hPSControls, IDC_SHOWBURSTMARKERS), FALSE);

	}
	else {
	  EnableWindow(GetDlgItem(hPSControls, IDC_BURST_BACK), TRUE);
      EnableWindow(GetDlgItem(hPSControls, IDC_BURST_NEXT), TRUE);
      EnableWindow(GetDlgItem(hPSControls, IDC_SHOWBURSTMARKERS), TRUE);
	}
}

/*******************************************************************************************************************
  Greys and inactivates count related windows - used to inactivate controls until data is loaded
 ******************************************************************************************************************/
void GreyOutControls(BOOL bState)
{
	if(bState) {
      EnableWindow(GetDlgItem(hPSControls, IDC_POSITION_TB), FALSE);
      EnableWindow(GetDlgItem(hPSControls, IDC_SEGTIME_SLIDER), FALSE);
      EnableWindow(GetDlgItem(hPSControls, IDC_PS1), FALSE);
      EnableWindow(GetDlgItem(hPSControls, IDC_PS2), FALSE);

	}
	else {
	  EnableWindow(GetDlgItem(hPSControls, IDC_POSITION_TB), TRUE);
      EnableWindow(GetDlgItem(hPSControls, IDC_SEGTIME_SLIDER), TRUE);
	  EnableWindow(GetDlgItem(hPSControls, IDC_PS1), TRUE);
      EnableWindow(GetDlgItem(hPSControls, IDC_PS2), TRUE);
	}
}


/*******************************************************************************************************************
  Toggles visibility of results window
 ******************************************************************************************************************/
void ToggleResultsWindow(HWND hwnd)	  
{
   if(bShowResultsWin) {
      bShowResultsWin = FALSE;
	  CheckMenuItem(GetMenu(hwnd), IDM_SHOWRESULTSWINDOW, MF_UNCHECKED);
      ShowWindow(WGGetPageWindow(pResultsPage), SW_HIDE); 
   }

   else {
	  bShowResultsWin = TRUE;
	  CheckMenuItem(GetMenu(hwnd), IDM_SHOWRESULTSWINDOW, MF_CHECKED);
	  ShowWindow(WGGetPageWindow(pResultsPage), SW_SHOW); 
  }
}


/*******************************************************************************************************************
  Closes out a PS data set 
 ******************************************************************************************************************/
void CloseDataSet(int set)
{
  PHOTON_STREAM *ps = GETPS(set);
  BURST *burst = GETBURST(set); 

  if(flags.bSet[set]) {
	 if(flags.bResultsReady[set]) {
	    FreeBurstMem(burst); 
	    flags.bResultsReady[set] = FALSE; 
	 }	 
	 FreePhotonStreamMem(ps);
	 flags.bSet[set] = FALSE;
	 
	 // reset binwidth and gray out controls
	 if((set == 0 && !flags.bSet[1])||(set == 1 && !flags.bSet[0])) {
		 flags.BinWidth = 0.0;
		 GreyOutBurstButtons(TRUE);
		 GreyOutControls(TRUE);
	 }
	 DisplayData();
  }   // flags if end
}

/*******************************************************************************************************************
  Shows the data table for the active PS
 ******************************************************************************************************************/
void ShowDataTable(int set)
{
 HGOBJ   hPlot = (set == 0) ? cd.hPlot1 : cd.hData2;

 WGSetDataFormat (cd.pGraph, hPlot, 9, 6);
 WGShowData (cd.pGraph, hPlot);
}		

/********************************************************************************************
  Called by WinProc when user clicks analyse bursts button
 ******************************************************************************************/
void BurstAnalysis(HWND hwnd, int set)
{
 int result; 
 PHOTON_STREAM *ps = GETPS(set);
 BURST *burst = GETBURST(set);
 char szErrorBoxTitle[] = "Burst Analysis parameter error"; 
 char szStr[256]; 

 if(!flags.bSet[set]) return;      // no data loaded.
 
 // Query user if there is burst data from a previous analysis.
 if(flags.bResultsReady[set]) {
	if(MessageBox(hwnd, "Overwrite Burst Data?", "Burst Data not saved", MB_YESNO) == IDNO) return;
 }	
 
 TurnOffBackGroundSelect(hwnd);
 TurnOffFCSSegmentSelect(hwnd);
 result = StartBurstAnalysis(hwnd, ps, burst, set); 
 if(result > NO_BURST_ANALYSIS_ERRORS) {
    strcpy(szStr, GetString(szStr, (UINT) result)); 
    MessageBox(hwnd, szStr, szErrorBoxTitle, MB_OK);
 }
 if(!bShowResultsWin) ToggleResultsWindow(hwnd);
 GreyOutBurstButtons(FALSE); 
}


char *GetString(char *szStr, UINT uID)
{
  LoadString(hInst, uID,szStr, 256); 
  return(szStr); 
}


/*******************************************************************************************************************
  Set up for burst analysis if data is loaded 
 ******************************************************************************************************************/
int StartBurstAnalysis(HWND hwnd, PHOTON_STREAM *ps, BURST *burst, int set)
{
 int result; 
 double mean, sd, min, max; 

 TurnOffBackGroundSelect(hwnd);
 
 // load the burst analysis parameters 
 GetUserSetParameters(); 
 burst->MinCountsPerBurst = init.MinCountsPerBurst; 
 burst->SubMinBins = init.SubMinBins;
 burst->TriggerCounts = init.TriggerCounts; 
 burst->BackGrdCounts = init.BackGrdCounts;
 burst->BinWidth = init.BinWidth;
 strcpy(burst->szFilename, ps->szFilename);             	
 burst->MaxBurstWidth = 500000;              // assume the maximum burst width of 0.5s
 
  // get memory
 result = GetBurstMem(burst, ps); 
 if(result > NO_BURST_ANALYSIS_ERRORS) return(result);  
 
 // analyze the data stream 
 result = AnalyzeData(ps, burst);
 if(result == BURST_ANALYSIS_ERROR)  return(BURST_ANALYSIS_ERROR);
  
 if(burst->numbursts == 0 ) return(NO_BURSTS_FOUND); 

 DisplayBurstHistograms(burst);
 
 flags.bResultsReady[set] = TRUE;
 flags.bResultsSaved[set] = FALSE;

 // get stat's on peak values (stats for other arrays are calculated in DisplayBurstHistogram())
 CalcMean(burst->Peak, 0, burst->numbursts - 1, &mean, &sd, &max, &min);
 burst->PeakMean = mean;
 burst->PeakSD = sd;
 burst->PeakMax = max;
 burst->PeakMin = min; 

 UpDateText(GETPS(active_ps), GETBURST(active_ps));
 return(0); 
}


/******************************************************************************************
  Called by WinProc - starts the data loading process
 *******************************************************************************************/
void LoadDataFile(HWND hwnd, int set)
{
  int result;
  char szBuf[256];
  PHOTON_STREAM *ps = GETPS(set);
 
  // get the current binwidth
  init.BinWidth = GetBinWidth(hPSControls); 
    
  // check that binwidth is the same if data is already loaded (if no data is loaded binwidth will be 0.0)
  if((flags.BinWidth > 0.0)&&(flags.BinWidth != init.BinWidth)) {    
  	  sprintf(szBuf, "Current binwidth setting is different then binwidth of loaded data - reset to %3.1f s", flags.BinWidth);
	  MessageBox(hwnd, szBuf, "Binwidth error", MB_OK);
	  return;
  }

  // check if a data if loaded in PS(set)
  if(flags.bSet[set]) {
      sprintf(szBuf, "PS%d array contains data - overwrite?", set+1);
	  result = MessageBox(hwnd, szBuf, "Data present", MB_YESNO);
      if(result == IDYES) 
		  FreePhotonStreamMem(ps);  
	  else 
		  return;
  }
  
  // check for loaded data and results 
  if(flags.bResultsReady[set] && !flags.bResultsSaved[set]) {
     result = MessageBox(hwnd, "Histogram and burst data not saved - continue? (This will overwrite data in the memory)", 
		                 "Data in memory", MB_YESNO);
     switch(result) {
	   case IDYES: 
		  if(set == 0) 
			  FreeBurstMem(&burst1); 
		  else
			  FreeBurstMem(&burst2); 
		  break;
	
	   case IDNO: return;
	 }
  }  
  
  OpenDataFile(hwnd, ps, init.BinWidth, set);
}

/****************************************************************************
 Called by WinProc - function carries out the user-slected PS math
 ***********************************************************************/
void DoPSMath(int command)
{
	
 // first check to be sure data is loaded:
 switch(command) {
 	case IDM_PS1_MINUS_C1: 
	case IDM_PS1_PLUS_C1:		  
	case IDM_PS1_A1_X_PS1:		  
	case IDM_PS1_PLUS_PS2:		  
	case IDM_PS1_MINUS_PS2:
	case IDM_PS1_ADD_PNOISE:
	    if(!flags.bSet[0]) return; 
	default: break;
  }

 switch(command) {
 	case IDM_PS2_MINUS_C2: 
	case IDM_PS2_PLUS_C2:		  
	case IDM_PS2_A2_X_PS2:		  
	case IDM_PS1_PLUS_PS2:		  
	case IDM_PS1_MINUS_PS2:
	case IDM_PS2_ADD_PNOISE:
	    if(!flags.bSet[1]) return; 
	default: break;
  }

 switch(command) {
 	case IDM_PS1_MINUS_C1: SubtractConstantFromDataSet(&ps1, init.c1); break;   // subtract C1 from PS1
	case IDM_PS1_PLUS_C1: AddConstantToDataSet(&ps1, init.c1);   break;         // add C1 to PS1
	case IDM_PS2_MINUS_C2: SubtractConstantFromDataSet(&ps2, init.c2); break;   // subtract C2 from PS2
	case IDM_PS2_PLUS_C2: AddConstantToDataSet(&ps2, init.c2); break;           // add C2 to PS2
	case IDM_PS1_A1_X_PS1: MultiplyDataSetByScalar(&ps1, init.a1); break;       // mult PS1 by init.a
	case IDM_PS2_A2_X_PS2: MultiplyDataSetByScalar(&ps2, init.a2); break;       // mult PS1 by init.a
	case IDM_PS1_PLUS_PS2: AddDataSets(&ps1, &ps2); break;                      // PS1 = PS1 + PS2
  	case IDM_PS1_MINUS_PS2: SubtractDataSets(&ps1, &ps2); break;                // PS1 = PS1 - PS2
	case IDM_PS1_ADD_PNOISE: AddPoisNoiseToDataSet(&ps1); break;                // PS1 = PS1 + Pois noise
  	case IDM_PS2_ADD_PNOISE: AddPoisNoiseToDataSet(&ps2); break;                // PS2 = PS2 + Pois noise
 }
 DisplayData();
}

/**************************************************************************************
  Called by WinProc() - begins the segmented autocorrelation analysis 
  *************************************************************************************/
void SegmentedAutocorrelation(HWND hwnd, int set)
{
 int result;
 AUTOCORRELATION *fcs = GETFCS(set);
 PHOTON_STREAM *ps = GETPS(set);  

 if(!flags.bSet[set]) { 
	 MessageBox(hwnd,"No data loaded", "Analyze Error", MB_OK); 
	 return; 
 }
 
 if(!flags.bResultsReady[set]) {
     MessageBox(hwnd,"Burst array empty - carry out a burst analysis first.", "Analyze Error",MB_OK);
	 return;
 }

 strcpy(fcs->szFilename, ps->szFilename); 
 fcs->burst_binwidth = ps->BinWidth;
 result = DialogBox(hInst,"SEGMENT_AUTOCORR", hwnd, (DLGPROC) SegmentedAutocorrelationProc); 
 if(result) return;  // user clicked cancel

 TurnOffBackGroundSelect(hwnd);
 if(!bShowResultsWin) ToggleResultsWindow(hwnd)	 ; 
 AutocorrelateSegments(fcs, set);
}					  



/**************************************************************************************
  Called by WinProc() - displays the photon count histogram for the active data set
  *************************************************************************************/
void DisplayPCHPlot(HWND hwnd, int set)
{
  char szBuf[64];
  PHOTON_STREAM *ps;

  if(!flags.bSet[set]) {
	 MessageBox(hwnd,"Data not loaded in selected PS.", "PCH Error",MB_OK);
	 return;
  }
  
  TurnOffBackGroundSelect(hwnd);
  ps = (set == 0) ? &ps1 : &ps2; 
  CalcPhotonCountHistogram(ps, &pch, 0, ps->nbins-1);
  sprintf(szBuf, "PS%d Photon Count Histogram", set+1);
  WGCreatePage("pch_page", hwnd, hInst, "Photon Count Histogram", (void *) InitPCHGraph,
		                  "PCHPAGEMENU", C_WHITE, MM_PROPORT, 0, PAGE_EXACT, 300, 100, 550, 350);  

}

/*******************************************************************************************************************
  Starts Help dialogs 
 ******************************************************************************************************************/
void HelpSystemDialog(HWND hwnd) 
{
  MessageBox(hwnd, "Help file not yet created", ">>> IMPEACH BUSH & CHENEY <<<", MB_OK);
  //WinHelp(hwnd, "burst.hlp", HELP_FINDER, 0);
}
	
/*******************************************************************************************************************
  Update CPS edit box in the PS control dialog box
 ******************************************************************************************************************/
void UpdateCPSEditbox(void)
{
 char szBuf[128];
 
 sprintf(szBuf, "%3.2f KHz", ps1.CPS);
 SendDlgItemMessage(hPSControls, IDC_SHOWCPS1, WM_SETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));
 sprintf(szBuf, "%u to %u", ps1.min, ps1.max);
 SendDlgItemMessage(hPSControls, IDC_SHOWMAX1, WM_SETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));

 sprintf(szBuf, "%3.2f KHz", ps2.CPS);
 SendDlgItemMessage(hPSControls, IDC_SHOWCPS2, WM_SETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));
 sprintf(szBuf, "%u to %u", ps2.min, ps2.max);
 SendDlgItemMessage(hPSControls, IDC_SHOWMAX2, WM_SETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));

 sprintf(szBuf, "%u", init.BackGrdCounts);
 SendDlgItemMessage(hPSControls, IDC_BACKGRDCOUNTS, WM_SETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));


}

/*******************************************************************************************************************
  Steps position slider 
 ******************************************************************************************************************/
void StepPositionTB(int direction)
{
 int newposition;
 HWND hwnd = GetFocus();
	
 // only step slider if it has the focus
 if(hwnd != GetDlgItem(hPSControls, IDC_POSITION_TB))  return;   
	
 newposition = SendDlgItemMessage(hPSControls, IDC_POSITION_TB, TBM_GETPOS, 0,0); 
 switch (direction) {
     case IDM_STEPFORWARD:
		 if(++newposition > (int) cd.NumSegments) return; 
		 break;
		 		 
	 case IDM_STEPBACKWARD: 
		 if(--newposition < 0) return; 
		 break; 
 }
 SendDlgItemMessage(hPSControls, IDC_POSITION_TB, TBM_SETPOS, TRUE, newposition); 
 CountsPositionTrackBar(); 
}


/*******************************************************************************************************************

 ******************************************************************************************************************/
void  SetupSegmentTimeTrackBar(void)
{
 cd.SegTimeTBMax = (WORD) ceil(log((double) cd.nbins/QC_SIZE)/LN2);
 SendDlgItemMessage(hPSControls, IDC_SEGTIME_SLIDER, TBM_SETRANGE, (WPARAM) TRUE, (LPARAM) MAKELONG(1, cd.SegTimeTBMax)); 
 SendDlgItemMessage(hPSControls, IDC_SEGTIME_SLIDER, TBM_SETPOS,   (WPARAM) TRUE, (LPARAM) 0); 
 SendDlgItemMessage(hPSControls, IDC_SEGTIME_SLIDER, TBM_SETTICFREQ, (WPARAM) 1, (LPARAM) 0); 
}


/*******************************************************************************************************************

 ******************************************************************************************************************/
void CountsPositionTrackBar(void)
{ 
 char szBuf[128];

 cd.SegNum = (DWORD) SendDlgItemMessage(hPSControls, IDC_POSITION_TB, TBM_GETPOS, 0,0); 
 sprintf(szBuf,"%u", cd.SegNum + 1);
 SendDlgItemMessage(hPSControls, IDC_POSITION, WM_SETTEXT, sizeof(szBuf),(LPARAM)((LPSTR) szBuf));
 DisplayData();
}
		  
/*******************************************************************************************************************

 ******************************************************************************************************************/
void  SegmentTimeTrackBar(void)
{
  double val; 
  char szBuf[128];

  val = (double) SendDlgItemMessage(hPSControls, IDC_SEGTIME_SLIDER, TBM_GETPOS ,0,0);
  val = cd.SegTimeTBMax - val;
  cd.BinsPerSegPt = (DWORD) floor(((double) cd.nbins / QC_SIZE)/pow(2.0, val)); //* QC_SIZE;
  cd.segtime =  cd.BinsPerSegPt * QC_SIZE* cd.binwidth * US_TO_SEC; 
  cd.DisplayedFrac = cd.segtime/cd.TraceLength; 
  cd.NumSegments = cd.nbins / (cd.BinsPerSegPt * QC_SIZE) ; 
  sprintf(szBuf,"%g s (%2.2f%%)", cd.segtime, cd.DisplayedFrac * 100.0); 
  SendDlgItemMessage(hPSControls, IDC_SEGMENT_TIME, WM_SETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));
  SetDisplayedTime();
}
  
/*******************************************************************************************************************
  Sets up the position slider - called by WinProc when the users clicks on Set button
 ******************************************************************************************************************/
void SetDisplayedTime(void)
{
  WORD ticks, numseg;
  char szBuf[128];

  numseg = (WORD) cd.NumSegments;
  ticks = (numseg < 20) ? 1 : numseg/10; 
  SendDlgItemMessage(hPSControls, IDC_POSITION_TB, TBM_SETRANGE, (WPARAM) TRUE, (LPARAM) MAKELONG(0, numseg-1)); 
  SendDlgItemMessage(hPSControls, IDC_POSITION_TB, TBM_SETPOS,   (WPARAM) TRUE, (LPARAM) 0); 
  SendDlgItemMessage(hPSControls, IDC_POSITION_TB, TBM_SETTICFREQ, (WPARAM) ticks, (LPARAM) 0); 
  sprintf(szBuf,"1"); 
  SendDlgItemMessage(hPSControls, IDC_POSITION, WM_SETTEXT, sizeof(szBuf), (LPARAM)((LPSTR) szBuf));
  DisplayData();
}

/*******************************************************************************************************************
  Sets the binwidth in the binwidth list box
 ******************************************************************************************************************/
#define NUMBINWIDTHS            11               // number of defined binwidths

void SetUpBinWidths(HWND hwnd, double binwidth)
{
 int i;

 // binwidth strings and int's in s:
 char szBinWidths[NUMBINWIDTHS][16] = {"0.1 s", "0.5 s", "1.0 s","2.0 s","5.0 s","10.0 s",
                                      "20.0 s","50.0 s","100.0 s", "250.0 s", "500 s"};
 double BinWidths[NUMBINWIDTHS] = {0.1, 0.5, 1.0, 2.0, 5.0, 10.0, 20.0, 50.0, 100.0, 250.0, 500.0}; 

 for(i=0;i<NUMBINWIDTHS;i++) SendDlgItemMessage(hwnd, IDC_BINWIDTH, CB_ADDSTRING, 0, (LPARAM) (LPCSTR) szBinWidths[i]);
 i = GetIndex(binwidth, BinWidths, NUMBINWIDTHS);
 SendDlgItemMessage(hwnd, IDC_BINWIDTH, CB_SETCURSEL, (WPARAM) i,  0);
}


/*******************************************************************************************************************
  Gets the binwidth from the binwidth list box
 ******************************************************************************************************************/

double GetBinWidth(HWND hwnd)
{
  double BinWidths[NUMBINWIDTHS] = {0.1, 0.5, 1.0, 2.0, 5.0, 10.0, 20.0, 50.0, 100.0, 250.0, 500.0}; 

  return(BinWidths[SendDlgItemMessage(hwnd, IDC_BINWIDTH, CB_GETCURSEL, 0L,  0L)]);
}

/*******************************************************************************************************************
  Sets the SG Filter width in the list box
 ******************************************************************************************************************/
#define NUM_SGFILTER_WIDTHS  5
void SetSGFilterWidths(HWND hwnd, int SGWidth)
{
 int i;
 WORD SGFilterWidth[NUM_SGFILTER_WIDTHS] = {5, 7, 9, 11, 13}; 
 char str[16];

 for(i=0;i<NUM_SGFILTER_WIDTHS;i++) {
	 sprintf(str, "%d", SGFilterWidth[i]); 
	 SendDlgItemMessage(hwnd, IDC_SGFILTER_WIDTH, CB_ADDSTRING, 0, (LPARAM) (LPCSTR) str);
 }
 i = GetIndexWord((WORD) SGWidth,  SGFilterWidth, NUM_SGFILTER_WIDTHS);
 SendDlgItemMessage(hwnd, IDC_SGFILTER_WIDTH, CB_SETCURSEL, (WPARAM) i,  0);
 
}



/*******************************************************************************************************************
  Gets the SG Filter order the SGFilter list box
 ******************************************************************************************************************/
int GetSGFilterWidth(HWND hwnd)
{
  int SGFilterWidth[NUM_SGFILTER_WIDTHS] = {5, 7, 9, 11, 13}; 

  return(SGFilterWidth[SendDlgItemMessage(hwnd, IDC_SGFILTER_WIDTH, CB_GETCURSEL, 0L,  0L)]);
}


/*******************************************************************************************************************
  Sets the active PS data set and toggles radio buttons
 ******************************************************************************************************************/
void SetActivePS(int set)	  
{
   if(set == 0) {
 	  CheckDlgButton(hPSControls, IDC_PS1, BST_CHECKED); 
 	  CheckDlgButton(hPSControls, IDC_PS2, BST_UNCHECKED); 
	  active_ps = set;
   }

   if(set == 1) {
	  CheckDlgButton(hPSControls, IDC_PS2, BST_CHECKED); 
 	  CheckDlgButton(hPSControls, IDC_PS1, BST_UNCHECKED); 
      active_ps = set;
  }
}

/*******************************************************************************************************************
  Toggles the check box for y autoscaling on counts plot
 ******************************************************************************************************************/
void ToggleCountsAutoScale(void)
{
  if(init.bAutoScale) {
    CheckDlgButton(hPSControls, IDC_AUTOSCALEY, BST_UNCHECKED); 
    init.bAutoScale = FALSE;
  }
  else {
    CheckDlgButton(hPSControls, IDC_AUTOSCALEY, BST_CHECKED); 
    init.bAutoScale = TRUE;
  }
}

/*******************************************************************************************************************
  Toggles the check box for histogram autoscaling
 ******************************************************************************************************************/
void ToggleHistAutoScale(void)
{
  if(init.bHistAutoScale) {
    CheckDlgButton(hPSControls, IDC_HISTAUTOSCALE, BST_UNCHECKED); 
    init.bHistAutoScale = FALSE;
  }
  else {
    CheckDlgButton(hPSControls, IDC_HISTAUTOSCALE, BST_CHECKED); 
    init.bHistAutoScale = TRUE;
  }
}
/*******************************************************************************************************************
  Toggles the burst marker flag 
 ******************************************************************************************************************/
void ToggleBurstMarkerFlag(void)
{
   if(cd.bShowBurstMarkers ) {
    CheckDlgButton(hPSControls, IDC_SHOWBURSTMARKERS, BST_UNCHECKED); 
    cd.bShowBurstMarkers  = FALSE;
  }
  else {
    CheckDlgButton(hPSControls, IDC_SHOWBURSTMARKERS, BST_CHECKED); 
    cd.bShowBurstMarkers = TRUE;
  }
}


/*******************************************************************************************************************
  Returns the new index from  the ps.counts array after the binwidth is changed and 
  data reloaded. Used by Autocorrelation, cross-correlation and segmented correlation functions 
  ******************************************************************************************************************/
int GetPSIndex(int index, PHOTON_STREAM *oldps, PHOTON_STREAM *newps)
{
 int oldn = oldps->nbins; 
 int newn = newps->nbins; 

 return((int) ((double) index * oldps->BinWidth/newps->BinWidth)) ;
}



/*******************************************************************************************************************
  Fills the init structure with current values - used by burst analysis proc and called before exiting to update 
  ini file
 ******************************************************************************************************************/
void GetUserSetParameters(void)
{
 char szBuf[256];
 int result; 

 SendDlgItemMessage(hPSControls, IDC_MINCOUNTS, WM_GETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));
 init.MinCountsPerBurst = (DWORD) atoi(szBuf);
 SendDlgItemMessage(hPSControls,IDC_SUBMINBINS, WM_GETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));
 init.SubMinBins = (DWORD) atoi(szBuf);
 SendDlgItemMessage(hPSControls,IDC_TRIGGER, WM_GETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));
 init.TriggerCounts = (DWORD) atoi(szBuf);
 
 if(init.MinCountsPerBurst < init.TriggerCounts) {
    init.MinCountsPerBurst = init.TriggerCounts;
    sprintf(szBuf,"%d",(int) init.MinCountsPerBurst);
    SendDlgItemMessage(hPSControls,IDC_MINCOUNTS, WM_SETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));
 }
 
 SendDlgItemMessage(hPSControls,IDC_BACKGRDCOUNTS, WM_GETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));
 init.BackGrdCounts = (DWORD) atoi(szBuf);
 init.BinWidth = GetBinWidth(hPSControls); 
 result = IsDlgButtonChecked(hPSControls, IDC_AUTOSUBTRACT);
 init.bBackgroundSubtract = (result == BST_CHECKED)? TRUE : FALSE;	
 
 result = IsDlgButtonChecked(hPSControls, IDC_HISTAUTOSCALE);
 init.bHistAutoScale = (result == BST_CHECKED)? TRUE : FALSE;
 
 result = IsDlgButtonChecked(hPSControls, IDC_SHOWBURSTMARKERS);
 init.bShowBurstMarkers = (result == BST_CHECKED)? TRUE : FALSE;

 init.SGFilterWidth = GetSGFilterWidth(hPSControls); 

}

