#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <time.h>    
#include "Burst.h"


#define INVALID 0x80000000
#define MTOV    0x40000000

// Get the data from a SPC file 
BOOL LoadSPCFile(HWND hwnd, char *szFilename,  PHCFILE_HEADER *fhd) 
{
  char szBuf[80];
  DWORD dwFileSizeHi,  dwFileSize, dwCNT, dwNumBytesRead, dwBufferSize;
  DWORD dwNumPhotons, dwBinWidth, dwNumBins, dwBinCounter, i, j; 
  LPVOID lpBuffer1, lpBuffer2;
  LPDWORD dwSPC; 
  LPDWORD dwTime;
  double TimeSum;
  HANDLE hFile;
  FILE *fp; 
  
  //GetHeaderInfo(hwnd, fhd);

  // open file for data read
  hFile = CreateFile(szFilename, GENERIC_READ, 0,NULL,OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
  if (hFile == INVALID_HANDLE_VALUE) return(FILE_OPEN_ERROR);
  
  HOURGLASS_CURSOR;  
  // load file
  dwFileSize = GetFileSize(hFile, &dwFileSizeHi);
  dwBufferSize = GetVirtualMemBufferSize(dwFileSize);
  lpBuffer1 = VirtualAlloc(NULL, dwBufferSize, MEM_COMMIT, PAGE_READWRITE);
  if(lpBuffer1 == NULL) return(MEM_ERROR);
  lpBuffer2 = VirtualAlloc(NULL, dwBufferSize, MEM_COMMIT, PAGE_READWRITE);
  if(lpBuffer2 == NULL) return(MEM_ERROR);


  // load the data
  ReadFile(hFile, lpBuffer1, dwFileSize, &dwNumBytesRead, NULL);
  if(dwNumBytesRead == 0) return(ZERO_BYTES_READ); 
  CloseHandle(hFile);

  // assign pointers 
  dwSPC  = lpBuffer1;
  dwTime = lpBuffer2;
  dwNumPhotons = dwFileSize/4;    // max possible photons 
  j = 0;
  i = 1;
  dwCNT = 0; 
  // start at 1 since the first value is the 50 ns
  do {
     if (dwSPC[i] & INVALID ) {
        if (dwSPC[i] & MTOV ) dwCNT += dwSPC[i] & 0xFFFFFFF;
     }
	 else {
        if (dwSPC[i] & MTOV ) dwCNT += 1;
        dwTime[j] = (dwCNT * 4096)  + (dwSPC[i] & 0xFFF);
        j++;
	 }
     i++;
  } while(i < dwNumPhotons);
  
  dwNumPhotons = j;
   
  // all done with dwSPC - reclaim some memory
  VirtualFree(lpBuffer1, 0, MEM_RELEASE);

  // dwTime now contains the number of 50 ns intervals between incoming photons, so 
  // calculate the total time of the trace:

  TimeSum = 0.0;
  for(i=0;i<dwNumPhotons;i++) TimeSum += (double) dwTime[i];
  dwNumBins = (DWORD) (0.05 * TimeSum);

  
  dwBinWidth = fhd->nBinWidth * 20;    // bin width in 50 ns units - 1 s min binsize
  dwBufferSize = GetVirtualMemBufferSize(4 * dwNumBins);
  lpBuffer1 = VirtualAlloc(NULL, dwBufferSize, MEM_COMMIT, PAGE_READWRITE);
  if(lpBuffer1 == NULL) return(MEM_ALLOC_ERROR);
  lpW = lpBuffer1; 
  
  // bin data
  i = j = 0;
  dwBinCounter = 0;
  do {
      while(dwTime[j] < dwBinCounter && j < dwNumPhotons) {
		  lpW[i]++;        
		  j++;
	  }
      dwBinCounter += dwBinWidth;
	  i++;
	 // j++; 
  } while(j<dwNumPhotons); 
  
  // save 
  strcpy(szBuf, szFilename);
  remove_fext(szBuf)  ;
  sprintf(szBuf, "%s.phc", szBuf);
  fp = fopen(szBuf,"wb");
  fwrite(lpW, sizeof(WORD), (size_t) dwNumBins, fp);
  fclose(fp);
  
  ARROW_CURSOR;
  return(NO_ERROR);
}


