/***************************************************************************
  BurstBatch.c   - Dialog box procedure and related functions for burst 
                   batch processing of a dir of data files
  *************************************************************************/

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "resource.h"
#include "burst.h"


char szLogBuffer[16000]; 


#define NUMCHAR  128

/********************************************************************************************
  Batch processing dialog box
  *****************************************************************************************/
BOOL CALLBACK BurstBatchProc (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
 static char szBuf[MAX_PATH];
 static char szDefaultDir[MAX_PATH];
 static BATCH_BURST bb;     // structure for batch processed burst data


 switch (message)    {
      case WM_INITDIALOG:
		  bb.bDirSelected = bb.bCompleted = bb.bRun = bb.bAbort = FALSE;
   		  if(init.bBatchFilter) CheckDlgButton(hDlg, IDC_BATCHFILTER,  BST_CHECKED);
		  if(init.bBatchFCS) CheckDlgButton(hDlg, IDC_BATCHFCS,  BST_CHECKED);
  		  if(init.bBatchSegFCS) CheckDlgButton(hDlg, IDC_BATCHSEGFCS,  BST_CHECKED);
		  if(init.bBatchCorCom) CheckDlgButton(hDlg, IDC_BATCHFILETYPE,  BST_CHECKED);
  		  if(init.bBatchEstBkgd) CheckDlgButton(hDlg, IDC_BATCHESTBKGD,  BST_CHECKED);
		  bb.NumFiles = 0;
	      bb.szFileNames = NULL; 
		  return (TRUE);
      
	  case WM_COMMAND:
	      switch (wParam) {
			case IDC_BATCHBROWSE: 
			     strcpy(szBuf, GetDirectoryFromBrowse(szBuf));
		         SendDlgItemMessage(hDlg, IDC_BATCHDIR, WM_SETTEXT, sizeof(szBuf), (LPARAM)((LPSTR) szBuf));
				 if(szBuf[0] != '\0') {
					 bb.bDirSelected = TRUE;
        	         strcpy(bb.szDir, szBuf);
     	             strcpy(bb.szSaveFileName, GetLastDirFromPath(szBuf));
		             SendDlgItemMessage(hDlg, IDC_BATCHSAVEAS, WM_SETTEXT, sizeof(szBuf), (LPARAM)((LPSTR) bb.szSaveFileName));
				 }
				 return(TRUE);

			case IDC_BATCHABORT:
				 bb.bRun = FALSE;
				 bb.bAbort = TRUE; 
		         EnableWindow(GetDlgItem(hDlg, IDC_BATCHCLOSE), TRUE);
    	         return (TRUE);
			
			case IDC_BATCHSTART:
				if(bb.bDirSelected) {
    	           GetCurrentDirectory(MAX_PATH, szDefaultDir);
   				   SetCurrentDirectory(bb.szDir);
                   init.bBatchFilter = (IsDlgButtonChecked(hDlg, IDC_BATCHFILTER))? TRUE:FALSE;
				   init.bBatchFCS = (IsDlgButtonChecked(hDlg, IDC_BATCHFCS))? TRUE:FALSE;
				   init.bBatchSegFCS = (IsDlgButtonChecked(hDlg, IDC_BATCHSEGFCS))? TRUE:FALSE;
                   init.bBatchCorCom = (IsDlgButtonChecked(hDlg, IDC_BATCHFILETYPE))? TRUE:FALSE;
				   bb.FileType = (IsDlgButtonChecked(hDlg, IDC_BATCHFILETYPE))? CORCOM_FILE : PHC_FILE;
				   GetUserSetParameters();
				   bb.SGFilterWidth = init.SGFilterWidth; 
		           bb.MinCountsPerBurst = init.MinCountsPerBurst;
		           bb.BinWidth = init.BinWidth;
		           bb.TriggerCounts = init.TriggerCounts;
				   bb.SubMinBins = init.SubMinBins; 

				   GetFilesForBatchProcessing(hDlg, &bb); // check dir for files and fill file list if they exist.
				   
				   if(bb.NumFiles > 0 && bb.szFileNames != NULL) {   // start thread
					   bb.hDlg = hDlg; 
					   bb.bRun = TRUE;
   				       bb.BackGrdCounts = (DWORD *) calloc(bb.NumFiles, sizeof(DWORD)); 
					   EnableWindow(GetDlgItem(hDlg, IDC_BATCHCLOSE), FALSE);
					   EnableWindow(GetDlgItem(hDlg, IDC_BATCHSTART), FALSE);
				       if(!bShowResultsWin) ToggleResultsWindow(GetParent(hDlg));
                       _beginthread(BatchProcessThread, 0, (LPVOID) &bb);
				   }
				   else {
					   MessageBox(hDlg, "No valid files were found in selected directory", "Batch Processing Error", MB_OK);
					   return(TRUE);
				   }
				}
				else {
					   MessageBox(hDlg, "Select a data directory for analysis", "Batch Processing Error", MB_OK);
				}
				return (TRUE);

		    case IDC_BATCHCLOSE: 
				 if(!bb.bCompleted && bb.bAbort) {
					MessageBox(hDlg, "Abort selected - Press Close again to exit dialogbox.", "Processing aborted", MB_OK); 
				 }
				 if(bb.bDirSelected) SetCurrentDirectory(szDefaultDir);
 				 FreeBatchBurstStruct(&bb);
				 EndDialog(hDlg, TRUE);
    	         return (TRUE);
			 
		}
    }
    return (FALSE);
}


/*************************************************************************************
  Determines how many files are in dir for batch processing 
 ************************************************************************************/
void GetFilesForBatchProcessing(HWND hDlg, BATCH_BURST *bb)
{
 int n;
 char szFileExt[32];
 char szBuf[256];

 switch(bb->FileType) {
 case PHC_FILE: strcpy(szFileExt,"*.phc"); break;
 case CORCOM_FILE: strcpy(szFileExt,"*.dat"); break;
 }
 
 bb->szFileNames = FindFiles(szFileExt, &n, NUMCHAR) ;
 if(n > 0) {
    bb->NumFiles = n;
    sprintf(szBuf, "Found %d %s files in %s\r\n", bb->NumFiles, szFileExt, bb->szDir);
	UpdateBatchStatus(bb->hDlg, szBuf); 
 }
}


/*************************************************************************************
  Thread function for batch processing mode
 ************************************************************************************/
void BatchProcessThread(PVOID pvoid)
{
  static int i; 
  static char szBuf[256], szStr[256];
  int ret; 
  BURST burst; 
  volatile BATCH_BURST *bb = (BATCH_BURST *) pvoid ;

  i = 0; 
  while(bb->bRun && i < bb->NumFiles) {
	// load file:
    sprintf(szBuf, "Loading file: %s\r\n", bb->szFileNames[i]);
    UpdateBatchStatus(bb->hDlg, szBuf);

    switch(bb->FileType) {
	   case PHC_FILE: 
		   break;
	   case CORCOM_FILE: 
		   ret = LoadCorrComFile(bb->szFileNames[i], (PHOTON_STREAM *) &bb->ps, bb->BinWidth) ;
           if(ret) {
		      sprintf(szBuf, "Error loading %s -- skipping this file.\r\n", bb->szFileNames[i]);
			  MessageBox(NULL, "szBuf", "Batch processing error", MB_OK); 
		   }
		   break;
	}

	// burst analysis
	if(init.bBatchFilter) {
	   SmoothPS((PHOTON_STREAM *) &bb->ps); 
       sprintf(szBuf, "Data file %s filtered with SG width %d\r\n", bb->szFileNames[i], bb->SGFilterWidth);
	   UpdateBatchStatus(bb->hDlg, szBuf);
	}

    if(init.bBatchEstBkgd) {
	   EstimateBackground((PHOTON_STREAM *) &bb->ps);
       bb->BackGrdCounts[i] = init.BackGrdCounts; 
	   sprintf(szBuf, "Data file %s background is %d\r\n", bb->szFileNames[i], init.BackGrdCounts);
	   UpdateBatchStatus(bb->hDlg, szBuf);
	}


    ret = StartBurstAnalysis(bb->hDlg, (PHOTON_STREAM *) &bb->ps, &burst, 0); 
    if(ret > NO_BURST_ANALYSIS_ERRORS) {
		sprintf(szBuf, "Data file %s: %s\r\n", bb->szFileNames[i], GetString(szStr, (UINT) ret));
        UpdateBatchStatus(bb->hDlg, szBuf);
	}
	else {
   		sprintf(szBuf, "Data file %s analysed\r\n", bb->szFileNames[i]);
		UpdateBatchStatus(bb->hDlg, szBuf);
	    // save burst data to the summed array
	    SaveSummedBurstData((BURST *) &bb->burst, &burst, (char *) bb->szSaveFileName); 
        DisplayBurstHistograms((BURST *) &bb->burst); 
	    // Do segmental FCS if selected
	    if(init.bBatchSegFCS) {
	      }
	}
	
	// Do trace FCS if selected 
	if (init.bBatchFCS) {
	
	
	}
	i++; 
  }
  
  
  if(i == bb->NumFiles) bb->bCompleted = TRUE; 
  bb->bRun = FALSE; 
  EnableWindow(GetDlgItem(bb->hDlg, IDC_BATCHCLOSE), TRUE); 
  EnableWindow(GetDlgItem(bb->hDlg, IDC_BATCHABORT), FALSE); 
  SaveBatchBurstInfoFile((BATCH_BURST *) bb); 
  UpdateBatchStatus(bb->hDlg, "Processing complete.\r\n");

  _endthread() ;

}


/**************************************************************************************
  Updates status box in batch dialog
  ************************************************************************************/
void UpdateBatchStatus(HWND hDlg, char *szStr)
{
  strcat(szLogBuffer, szStr); 
  SendDlgItemMessage(hDlg, IDC_BATCHSTATUS, WM_SETTEXT, strlen(szLogBuffer) + 1, (LPARAM)((LPSTR) szLogBuffer));
  SendDlgItemMessage(hDlg, IDC_BATCHSTATUS, LOWORD(WM_VSCROLL), SB_BOTTOM, 0);
}



/**************************************************************************************
  Saves and compiles the running burst data from analysis 
  bb->burst is reallocated as needed to fit the total data
  ************************************************************************************/
void SaveSummedBurstData(BURST *dest, BURST *source, char *szBase)
{
  static int num = 0; 
  static int oldnum = 0; 
  int i, n;
  char szFileName[128];
  FILE *fp; 

  num += source->numbursts;
  n = source->numbursts; 

  dest->Burst = (double *) realloc(dest->Burst, sizeof(double) * num );
  dest->Interval = (double *) realloc(dest->Interval, sizeof(double) * num);
  dest->Width = (double *) realloc(dest->Width, sizeof(double) * num);
  dest->Peak = (double *) realloc(dest->Peak, sizeof(double) * num);
   
  for(i=0;i<n;i++) {
      dest->Burst[i + oldnum] = source->Burst[i];
      dest->Peak[i + oldnum] = source->Peak[i];
      dest->Width[i + oldnum] = source->Width[i];
      dest->Interval[i + oldnum] = source->Interval[i];
  }
  dest->numbursts = oldnum = num; 

  sprintf(szFileName, "%s_bursts.txt", szBase);
  fp = fopen(szFileName, "wt"); 
   
  fprintf(fp, "%s\t%s\t%s\t%s\t%s\n", "no.", "photons", "peak", "width", "interval");
  for(i=0;i<oldnum;i++) {
      fprintf(fp, "%d\t%d\t%d\t%d\t%f\n", i+1, (int) dest->Burst[i], (int) dest->Peak[i], (int) dest->Width[i],  dest->Interval[i]); 
  }
  fclose(fp); 
 }

/**************************************************************************************
  Save a text file that contains batch analysis info
 ************************************************************************************/
void SaveBatchBurstInfoFile(BATCH_BURST *bb)
{
 FILE *fp; 
 char szFileName[128];
 int i; 

 sprintf(szFileName, "%s_info.txt", bb->szSaveFileName);
 fp = fopen(szFileName, "wt"); 
 
 fprintf(fp, "Total number of bursts found = %d\n", bb->burst.numbursts); 
 fprintf(fp, "Min. photons/burst = %d\n", bb->MinCountsPerBurst); 
 fprintf(fp, "Allowed sub-min bins = %d\n", bb->SubMinBins); 
 fprintf(fp, "Trigger value = %d\n", bb->TriggerCounts); 
 fprintf(fp, "Binwidth (s) = %f\n", bb->BinWidth); 
 if(init.bBatchFilter) 
	  fprintf(fp, "SG Filter width = %d\n", bb->SGFilterWidth); 
 else
	  fprintf(fp, "%s\n", "Data sets not SG Filtered");
 
 fprintf(fp, "Files:\n");
 
 for(i=0;i<bb->NumFiles;i++) fprintf(fp, "%s (background level used = %d)\n", bb->szFileNames[i], bb->BackGrdCounts[i]); 

 fprintf(fp, "%s\n", "Process log:");
 fprintf(fp, "%s", szLogBuffer); 
 
}


/*******************************************************************************
  Function for running the Browse dialog window  - inserted from BrowseTest.c
 ***************************************************************************/
int CALLBACK  BrowseCallbackProc(HWND hwnd,UINT uMsg,LPARAM lp, LPARAM pData) 
{
 TCHAR szDir[MAX_PATH];

 switch(uMsg) {
    case BFFM_INITIALIZED: 
		 if (GetCurrentDirectory(sizeof(szDir)/sizeof(TCHAR),szDir)) {
             // WParam is TRUE since you are passing a path.
             // It would be FALSE if you were passing a pidl.
             SendMessage(hwnd,BFFM_SETSELECTION,TRUE,(LPARAM)szDir);
		 }
          break;
            
    case BFFM_SELCHANGED: 
         // Set the status window to the currently selected path.
         if (SHGetPathFromIDList((LPITEMIDLIST) lp ,szDir)) {
             SendMessage(hwnd,BFFM_SETSTATUSTEXT,0,(LPARAM)szDir);
         }
         break;
            
    default: break;
 }
 return 0;
}

/**************************************************************************************
  Returns the selected directory from the Browse box
  ************************************************************************************/
char *GetDirectoryFromBrowse(char *szReturnedDir) 
{
  BROWSEINFO bi;
  TCHAR szDir[MAX_PATH];
  LPITEMIDLIST pidl;
  LPMALLOC pMalloc;

  if (SUCCEEDED(SHGetMalloc(&pMalloc))) {
     ZeroMemory(&bi,sizeof(bi));
     bi.hwndOwner = NULL;
     bi.pszDisplayName = 0;
     bi.pidlRoot = 0;
     bi.ulFlags = BIF_RETURNONLYFSDIRS | BIF_STATUSTEXT;
     bi.lpfn = BrowseCallbackProc;

     pidl = SHBrowseForFolder(&bi);
     if (pidl) {
         if(SHGetPathFromIDList(pidl, szDir)) 
			 strcpy(szReturnedDir, szDir);
		 else 
			 szReturnedDir[0] = '\0';
		 
		 pMalloc->lpVtbl->Free(pMalloc,pidl);
         pMalloc->lpVtbl->Release(pMalloc);
     }
  }
  
  return(szReturnedDir);
}

/**************************************************************************************
  Returns the last dir from a dir path 
 ************************************************************************************/
char *GetLastDirFromPath(char *szPath)
{
 char szBuf[MAX_PATH];
 char *pdest; 

 strcpy(szBuf, szPath);
 pdest = strrchr(szBuf, '\\');
 strcpy(szPath, pdest + 1); 
 return(szPath); 
}


/*******************************************************************************
 Gets all the files in the target directory of the form szFileType
 *****************************************************************************/
#define MAXFILES   512
char **FindFiles(char *szFileType, int *num, int NumChar) 
{
  WIN32_FIND_DATA FindFileData;
  HANDLE hFind;
  char **szTemp;
  char **szFileNames = NULL; 
  int i, n;
  
  szTemp = StringArray(MAXFILES, NumChar);    // get some mem for file names
  
  hFind = FindFirstFile(szFileType, &FindFileData);
  if (hFind == INVALID_HANDLE_VALUE) return(szFileNames);  // no file with right ext found - return NULL ptr
  
  // one file found with right name - keep looking
  strcpy(szTemp[0], FindFileData.cFileName);
  n = 1;
  while(FindNextFile(hFind, &FindFileData) && n<MAXFILES) {
            strcpy(szTemp[n], FindFileData.cFileName);
			n++;
	  }

  FindClose(hFind);
  *num = n; 
  szFileNames = StringArray(n, NumChar);
  for(i=0;i<n;i++) strcpy(szFileNames[i], szTemp[i]);
  FreeStringArray(szTemp, MAXFILES);
  return (szFileNames);
}

/**************************************************************************************
  Create 2D string array
 ************************************************************************************/
char **StringArray(int NumStrings, int NumChar)
{
 int i;
 char **mptr;

 mptr=(char **) calloc(NumStrings, sizeof(char *));
 if (mptr==NULL) return(NULL);
 for(i=0;i<NumStrings;i++) {
     mptr[i]=(char *) calloc(NumChar, sizeof(BYTE));
     if (mptr[i]==NULL) return(NULL);
 }
 return(mptr);
}

/**************************************************************************************
  Free 2D string array
 ************************************************************************************/
void FreeStringArray(char **mptr, int NumStrings)
{
 int i;

 for(i=0;i<NumStrings;i++) if(mptr[i]) free(mptr[i]);
 if(mptr) free(mptr);
}

/**************************************************************************************
  Cleans up batch burst struct
  ************************************************************************************/
void FreeBatchBurstStruct(BATCH_BURST *bb)
{
 if(bb->szFileNames) FreeStringArray(bb->szFileNames, bb->NumFiles);
 bb->szFileNames = NULL; 
 if(bb->ps.counts) VirtualFree(bb->ps.counts, 0, MEM_RELEASE);  
 bb->ps.counts = NULL; 
 if(bb->burst.Burst) free(bb->burst.Burst);
 bb->burst.Burst = NULL;
 if(bb->burst.Interval) free(bb->burst.Interval);
 bb->burst.Interval = NULL; 
 if(bb->burst.Peak) free(bb->burst.Peak);
 bb->burst.Peak = NULL; 
 if(bb->burst.Width) free(bb->burst.Width);
 bb->burst.Width = NULL; 
 if(bb->BackGrdCounts) free(bb->BackGrdCounts); 
 bb->BackGrdCounts = NULL; 
 bb->BinWidth = 0.0;
 bb->MinCountsPerBurst = 0; 
 bb->TriggerCounts = 0; 
 bb->bCompleted = bb->bRun = bb->bDirSelected = bb->bAbort = FALSE; 

}

