//
//  BurstNoise.c   - noise routines for BURST.C
//

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>

#define PI                      3.1415926535


/* returns a uniform deviate from a shuffled array */
double RanNum(void)
{
 static double y, maxran, v[100];
 static int iff=0;
 int i;

 /* initialize array on first pass thru function */
 if (iff == 0) {
  	 maxran = (double) RAND_MAX;
     iff=1;
	 srand((unsigned) time(NULL));
	 for (i=0;i<100;i++) v[i]= (double) rand();
	 y = (double) rand();
	 return(y/maxran);
	 }
 
 i = (int) (99.0 * y/maxran);
 y = v[i];
 v[i] = (double) rand();
 return (y/maxran);
}

/* returns an exponential deviate distance */
double ExpDev(void)
{
 double r;

 do { r = RanNum(); } while (r == 0.0);
 return (-log(r));
}

double GammLn(double xx)
{
 int j;
 double x, tmp, ser;
 static double cof[6] = {76.18009173, -86.50532033, 24.01409822,
	                     -1.231739516, 0.120858003e-2, -0.536382e-5};

 x = xx - 1.0;
 tmp = x + 5.5;
 tmp -= (x + 0.5) * log(tmp);
 ser = 1.0;
 for(j=0;j<6;j++) {
	 x += 1.0;
	 ser += cof[j]/x;
 }
 return(-tmp + log(2.50662827465 * ser));
}

double GamDev(int ia)
{
 int j;
 double am, e, s, v1, v2, x,y;

 if(ia<6) {
	 x = 1.0;
	 for(j=0;j<ia;j++) x *= RanNum();
	 x = -log(x);
 }
 else {
	 do {
		 do {
			 do {
				 v1 = 2.0 * RanNum() - 1.0;
				 v2 = 2.0 * RanNum() - 1.0;
			 }  while ((v1 * v1 + v2 * v2) > 1.0);
             y = v2/v1;
		     am = ia - 1;
			 s = sqrt(2.0 * am + 1.0);
			 x = s * y + am;
		 } while (x <= 0.0);
		 e = (1.0 * y * y) * exp(am * log(x/am) - s * y);
	 } while (RanNum() > s);
 }
 return(x);
}

double PoisDev(double xm)
{
 static double sq, alxm, g, oldm = -1.0;
 double em, t, y;

 if(xm < 12.0) {
	 if(xm != oldm) {
	    oldm = xm;
	    g = exp(-xm);
	 }
     em = -1.0;
     t = 1.0;
     do {
	   em += 1.0;
	   t *= RanNum();
	 } while (t > g);
 }
 else {
     if(xm != oldm) {
		 oldm = xm;
		 sq = sqrt(2.0 * xm);
		 alxm = log(xm);
		 g = xm * alxm - GammLn(xm + 1.0);
	  }
	  do {
		 do {
			 y = tan(PI * RanNum());
			 em = sq * y + xm;
		 } while(em < 0.0);
		 em = floor(em);
		 t = 0.9 * (1.0 + y * y) * exp(em * alxm - GammLn(em + 1.0) - g);
	 } while(RanNum() > t);
 }
 return(em);
}

double GaussDev(void)
{
 static int iset = FALSE;
 static double gset;
 double fac, r, v1, v2;

 if(iset == FALSE) {
	 do {
		 v1 = 2.0 * RanNum() - 1.0;
         v2 = 2.0 * RanNum() - 1.0;
         r = v1 * v1 + v2 * v2;
	 } while (r >= 1.0 || r ==0.0);
	 fac = sqrt(-2.0 * log(r) / r);
	 gset = v1 * fac;
	 iset = TRUE;
	 return(v2 * fac);
 }
 else {
	 iset = FALSE;
	 return(gset);
 }
}


BOOL FlipCoin(void)
{
 double a = RanNum();
 
 if(a < 0.5) 
	return(TRUE); 
 else 
    return(FALSE);
}

