/***********************************************************************
 BurstSGFilter.C - Savitzky-Golay code
 
 SavitzkyGolaySmooth -  Reduces the amount of noise in a data set using
 the Savitzky-Golay technique of simplified least squares smoothing and differentiation.  
 **********************************************************************/
#include <windows.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "burst.h"

/******************************************************************************************************************************
  Smooth the data set with SG Filter 
 ***************************************************************************************************************************/
void SGSmoothData(int set)
{
 int result; 
 char szStr[128];
 PHOTON_STREAM *ps = GETPS(set); 

 if(!flags.bSet[set]) return; 

 init.SGFilterWidth = GetSGFilterWidth(hPSControls);

 if(ps->SGFilterPasses) {
	 if(ps->SGFilterPasses == 1) 
	     sprintf(szStr, "Data has already been filtered (%d pass)", ps->SGFilterPasses);
	 else 
		 sprintf(szStr, "Data has already been filtered (%d passes)", ps->SGFilterPasses);
	 result = MessageBox(NULL, szStr, "Continue with smoothing?", MB_YESNO); 
	 if(result == IDNO) return;
 }

 if(SmoothPS(ps)) {
     UpdateCPSEditbox();
	 DisplayData();
 }
}



BOOL SmoothPS(PHOTON_STREAM *ps)
{
 float *dataset, *smoothdata;
 DWORD i, n, NumCounts;
 BOOL bRet; 
 WORD  Min; 

 n = ps->nbins;
 dataset = (float *) GetVirtualMem(n * sizeof(float)); 
 smoothdata = (float *) GetVirtualMem(n * sizeof(float)); 
 
 for(i=0;i<n;i++) dataset[i] = (float) ps->counts[i];
 
 bRet = SavitzkyGolaySmooth(dataset, n, init.SGFilterWidth, smoothdata, FALSE); 
 if (bRet) {   
    for(i=0;i<n;i++) ps->counts[i] = (WORD) smoothdata[i];
    ps->max =  GetMaxWord(ps->counts, ps->nbins, &NumCounts, &Min);
    ps->min = Min;
	ps->SGFilterPasses++;
 }
 

 VirtualFree(dataset, 0, MEM_RELEASE);  
 VirtualFree(smoothdata, 0, MEM_RELEASE);  
 smoothdata = dataset = NULL; 
 if(bRet) 
	return(TRUE); 
 else 
	return (FALSE); 
}




/* 
  FUNCTION  SavitzkyGolaySmooth
  note - parameter smoothnum is the index of the SGFilterWidth array 
     0 means 5 pts wide
	 1 means 7 pts wide
	 2 means 9 pts wide
	 3 means 11 pts wide
	 4 means 13 pts wide
*/

BOOL SavitzkyGolaySmooth(float *dataset, int n, int numcoef, float *smoothdata, BOOL bZeroArray)
{
  int i, j, k, smoothdeg, start, end, smoothnum;
  int normcoef[5] = {35,21,231,429,143}; 
  int smoothcoef[5][13] = { {-3,12,17,12,-3,0,0,0,0,0,0,0,0},          // coeffs for width = 5 pts
							{-2,3,6,7,6,3,-2,0,0,0,0,0,0},             // coeffs for width = 7 pts
							{-21,14,39,54,59,54,39,14,-21,0,0,0,0},    // coeffs for width = 9 pts
							{-36,9,44,69,84,89,84,69,44,9,-36,0,0},    // ceoffs for width = 11 pts
							{-11,0,9,16,21,24,5,24,21,16,9,0,-11}};    // ceoffs for width = 13 pts

  


  if(bZeroArray) for(i=0; i<n; i++) smoothdata[i] = 0.0;
  smoothnum = ((numcoef - 3)/ 2) - 1;
  if((smoothnum < 0) || (smoothnum > 4)) return FALSE;  // error
  smoothdeg = (numcoef - 1) / 2;
  start = smoothdeg;
  end = n - smoothdeg - 1;
 
  for(i=start; i <= end; i++) {
	  for(j=0; j<numcoef; j++) smoothdata[i] = smoothdata[i] + dataset[i-smoothdeg+j] * smoothcoef[smoothnum][j];
	  smoothdata[i] = smoothdata[i]/normcoef[smoothnum];
  }

  j = smoothdeg - 1;
  k = n - smoothdeg;
  for(i=0; i<smoothdeg; i++) {
	  smoothdata[j] = smoothdata[j+1];
	  smoothdata[k] = smoothdata[k-1];
	  j--;
	  k++;
  }
 return TRUE; 
}


